package edu.ntnu.stud.chaosgame.view.modificationpopups;

import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;
import javafx.stage.Modality;
import javafx.stage.Stage;

/**
 * Abstract class for popups.
 */
public abstract class AbstractPopup {

  /**
   * The stage for the popup.
   */
  protected Stage popupModifyStage;

  /**
   * The layout of the popup.
   */
  protected VBox layout;

  /**
   * The scene of the popup.
   */
  protected Scene scene;

  /**
   * The text field for the name.
   */
  protected TextField nameTextField;

  /**
   * The text field for the minimum x value.
   */
  protected TextField minXTextField;

  /**
   * The text field for the minimum y value.
   */
  protected TextField minYTextField;

  /**
   * The text field for the maximum x value.
   */
  protected TextField maxXTextField;

  /**
   * The text field for the maximum y value.
   */
  protected TextField maxYTextField;

  /**
   * The update button.
   */
  protected Button updateButton;

  /**
   * The layout for the bottom part of the popup.
   */
  protected VBox bottomLayout;

  /**
   * Constructor for AbstractPopup.
   *
   * @param layout the layout of the popup.
   */
  public AbstractPopup(VBox layout) {
    this.popupModifyStage = new Stage();
    this.popupModifyStage.initModality(Modality.APPLICATION_MODAL);
    this.popupModifyStage.setAlwaysOnTop(true);
    this.layout = layout;
    this.scene = new Scene(layout);
    this.popupModifyStage.setScene(scene);

    initializeCommonElements();
  }

  /**
   * Initialize the common elements of the popup.
   */
  private void initializeCommonElements() {
  nameTextField = new TextField();
  HBox nameBox = new HBox(new Label("Name: "), nameTextField);

  minXTextField = createNumericTextField();
  minYTextField = createNumericTextField();
  maxXTextField = createNumericTextField();
  maxYTextField = createNumericTextField();
  updateButton = new Button("Update");

  minXTextField.setPrefWidth(50);
  minYTextField.setPrefWidth(50);
  maxXTextField.setPrefWidth(50);
  maxYTextField.setPrefWidth(50);

  GridPane minCoordsGrid = new GridPane();
  minCoordsGrid.setHgap(10);
  minCoordsGrid.add(new Label("Min Coordinates: "), 0, 0);
  minCoordsGrid.add(minXTextField, 1, 0);
  minCoordsGrid.add(minYTextField, 2, 0);

  GridPane maxCoordsGrid = new GridPane();
  maxCoordsGrid.setHgap(10);
  maxCoordsGrid.add(new Label("Max Coordinates: "), 0, 0);
  maxCoordsGrid.add(maxXTextField, 1, 0);
  maxCoordsGrid.add(maxYTextField, 2, 0);

  bottomLayout = new VBox(10, minCoordsGrid, maxCoordsGrid, updateButton);
  bottomLayout.setAlignment(Pos.CENTER); // Center align the children
  bottomLayout.setPadding(new Insets(10));

  VBox.setVgrow(bottomLayout, Priority.ALWAYS);

  layout.getChildren().add(0, nameBox);
}

  /**
   * Create a numeric text field.
   *
   * @return the numeric text field.
   */
  protected TextField createNumericTextField() {
    TextField textField = new TextField();
    textField.textProperty().addListener((observable, oldValue, newValue) -> {
      if (!newValue.matches("-?\\d*(\\.\\d*)?")) { // TODO: replace with Formatter
        textField.setText(oldValue);
      }
    });
    return textField;
  }

  /**
   * Initialize the layout of the popup.
   *
   * @return the layout of the popup.
   */
  public static VBox initializeLayout() {
    VBox layout = new VBox();
    layout.setPadding(new Insets(10));
    layout.setSpacing(10);
    return layout;
  }

  /**
   * Display the popup.
   */
  public void display() {
      popupModifyStage.show();
  }

  /**
   * Get the stage of the popup.
   *
   * @return the stage of the popup.
   */
  public Stage getPopupModifyStage() {
      return popupModifyStage;
  }

  /**
   * Get the text field for the name.
   *
   * @return the text field for the name.
   */
  public TextField getNameTextField() {
      return nameTextField;
  }

  /**
   * Get the text field for the minimum x value.
   *
   * @return the text field for the minimum x value.
   */
  public TextField getMinXTextField() {
      return minXTextField;
  }

  /**
   * Get the text field for the minimum y value.
   *
   * @return the text field for the minimum y value.
   */
  public TextField getMinYTextField() {
      return minYTextField;
  }

  /**
   * Get the text field for the maximum x value.
   *
   * @return the text field for the maximum x value.
   */
  public TextField getMaxXTextField() {
      return maxXTextField;
  }

  /**
   * Get the text field for the maximum y value.
   *
   * @return the text field for the maximum y value.
   */
  public TextField getMaxYTextField() {
      return maxYTextField;
  }

  /**
   * Get the update button.
   *
   * @return the update button.
   */
  public Button getUpdateButton() {
      return updateButton;
  }
}

